<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\DataLayer;

use Flat101\EnhancedEcommerce\Api\Data\DataLayerInterface;
use Flat101\EnhancedEcommerce\Model\UnsetProperty;
use Flat101\EnhancedEcommerce\Model\Session;
use Magento\Checkout\Model\Session as CheckoutSession;
use Magento\Sales\Model\Order\Item;
use Flat101\EnhancedEcommerce\Plugin\ShippingInformation;

class PurchaseView implements DataLayerInterface
{
    use UnsetProperty;

    public const CODE = 'purchase-view';

    private const EVENT = 'transactionComplete';

    private const EXCLUDE_FIELDS = [
        'eventCategory',
        'eventAction',
        'eventLabel'
    ];

    /** @var Session */
    private $session;

    /** @var CheckoutSession */
    private $checkoutSession;

    /** @var Generator\Product */
    private $productGenerator;

    public function __construct(
        Session $session,
        CheckoutSession $checkoutSession,
        Generator\Product $productGenerator
    ) {
        $this->session = $session;
        $this->checkoutSession = $checkoutSession;
        $this->productGenerator = $productGenerator;
    }

    public function getCode(): string
    {
        return static::CODE;
    }

    public function getLayer()
    {
        $order = $this->checkoutSession->getLastRealOrder();

        $productItems = [];

        /** @var Item $item */
        foreach ($order->getAllVisibleItems() as $item) {
            $productItems[] = $this->productGenerator->generate($item->getProduct(), $item->getQtyOrdered(), $item);
        }

        $purchaseOrderDto = new Dto\Purchase\Order();
        $purchaseOrderDto->id = $order->getIncrementId();
        $purchaseOrderDto->revenue = floatval($order->getGrandTotal());
        $purchaseOrderDto->tax = floatval($order->getTaxAmount());
        $purchaseOrderDto->shipping = floatval($order->getShippingAmount());
        $purchaseOrderDto->coupon = $order->getCouponCode() ?: '';

        $purchaseDetailsDto = new Dto\EcommerceDetails();
        $purchaseDetailsDto->products = $productItems;
        $purchaseDetailsDto->actionField = $purchaseOrderDto;

        $purchaseDto = new Dto\Purchase\Purchase();
        $purchaseDto->purchase = $purchaseDetailsDto;
        $purchaseDto->currencyCode = $order->getOrderCurrencyCode();

        $ecommerce = new Dto\Ecommerce();
        $ecommerce->ecommerce = $purchaseDto;
        $ecommerce->event = self::EVENT;

        $this->unset($ecommerce, self::EXCLUDE_FIELDS);

        return $ecommerce;
    }
}
