define([
    'jquery',
    'underscore',
    'Magento_Customer/js/customer-data',
], function ($, _, storage) {
    'use strict';

    const storageName = 'analyzer-data';

    let stateCart = { };

    function initBefore() {
        window.dataLayer = window.dataLayer || [];
        dataLayer.push({ ecommerce: null });  // Clear the previous ecommerce object.
    }

    function initCartDataLayer(event, action, product, currency = null) {
        initBefore();
        if (_.isArray(product) && product.length > 0) {
            cleanData(product[0]);
        }

        product.forEach(function (item) {
            if (item.hasOwnProperty('item_id')) {
                event = 'add_to_cart';
            }
            cleanData(item);
            dataLayer.push({
                'event': event,
                'ecommerce': {
                    'currencyCode': currency,
                    [action]: {
                        'products': item
                    }
                }
            });
        });
    }

    function dataLayerPush(data) {
        initBefore();
        cleanData(data);
        if (_.has(data, 'ecommerce')) {
            cleanData(data.ecommerce);
        }

        if (_.has(data.ecommerce, 'actionField')) {
            cleanData(data.ecommerce.actionField);

            if (_.has(data.ecommerce.checkout, 'actionField')) {
                cleanData(data.ecommerce.checkout.actionField);
            }

            if (_.has(data.ecommerce.checkout, 'products')) {
                data.ecommerce.checkout.products.forEach((prod) =>cleanData(prod));
            }
        }


        // Avoid possible empty Product List Datalayer for PDP
        if (_.has(data, 'event') && data.event !== 'productList') {
            dataLayer.push(data);
        } else if (_.has(data.ecommerce, 'impressions') && _.isArray(data.ecommerce.impressions) && data.ecommerce.impressions.length > 0) {
            dataLayer.push(data);
        }
    }

    /**
     * Clean null keys
     * @param data
     */
    function cleanData(data) {
        Object.keys(data).forEach(key => data[key] === null && delete data[key]);
    }

    /**
     * @param {Object} config
     */
    return function (config) {
        if (_.has(config, 'dataLayer') && _.isArray(config.dataLayer) && config.dataLayer.length > 0) {
            config.dataLayer.forEach((item) => {
                dataLayerPush(item);
            });
        }

        let analyzerData = storage.get(storageName);

        analyzerData.subscribe((dataObject) => {
            if (!_.isObject(dataObject)) {
                return;
            }

            let cartValidate = _.has(dataObject, 'cart') && _.isArray(dataObject.cart) && dataObject.cart.length > 0;
            if (cartValidate && !_.isEqual(stateCart, dataObject.cart)) {
                initCartDataLayer('addToCart', 'add', dataObject.cart, config.currency);

                stateCart = dataObject.cart;
            }

            if (_.has(dataObject, 'removeCart') && _.isArray(dataObject.removeCart) && dataObject.removeCart.length > 0) {
                initCartDataLayer('removeFromCart', 'remove', dataObject.removeCart, config.currency);
            }

            if (_.has(dataObject, 'checkoutSteps') && _.isArray(dataObject.checkoutSteps) && dataObject.checkoutSteps.length > 0) {
                _.each(dataObject.checkoutSteps, dataLayerPush);
            }

            if (_.has(dataObject, 'behavioralEvents') && _.isArray(dataObject.behavioralEvents) && dataObject.behavioralEvents.length > 0) {
                _.each(dataObject.behavioralEvents, dataLayerPush);
            }
        });
    };
});
